# -*- coding:utf-8 -*-
import collections
from types import FunctionType
from typing import (
    Optional,
    Union,
    Literal,
    ClassVar,
    Dict,
    List,
    Mapping,
    Collection,
    Any,
    NoReturn,
    Final,
    Tuple,
    Sequence,
    Counter,
    get_origin, Iterable, Set,
)
from .constants import _DATACLASS_FIELDS_NAME
from .utils import isinstance_safe, issubclass_safe


class TypeParser:
    """类型解析器"""

    """必须有子类型的类型"""
    __required_subtype_types__: ClassVar[list] = [Optional, Union, ClassVar]

    """希望有子类型的类型"""
    __hope_subtype_types__: ClassVar[list] = [Dict, List, Mapping, Collection, Tuple, Sequence, Counter]

    def is_any(self, value: Any) -> bool:
        """是否任意类型的"""
        if value is None:
            return False
        return value is Any

    def is_optional(self, annotation) -> bool:
        """是否可为空"""
        if annotation is None:
            return False
        return annotation is Optional or getattr(annotation, '_name', None) == 'Optional' or annotation is Any

    def is_no_return(self, annotation) -> bool:
        """是否为不返回类型"""
        if annotation is None:
            return False
        return annotation is NoReturn

    def is_union(self, annotation) -> bool:
        """是否联合类型的"""
        if annotation is None:
            return False
        return get_origin(annotation) is Union or getattr(get_origin(annotation), '_name', None) == 'Union'

    def is_literal(self, annotation) -> bool:
        """是否字面量类型的"""
        if annotation is None:
            return False
        origin = get_origin(annotation)
        return origin is Literal or getattr(origin, '_name', None) == 'Literal'

    def is_final(self, value) -> bool:
        """是否最后类型的"""
        if value is None:
            return False
        return value is Final or getattr(get_origin(value), '_name', None) == 'Final'

    def is_class_var(self, value) -> bool:
        """是否为类共享类型的"""
        if value is None:
            return False
        return value is ClassVar or getattr(get_origin(value), '_name', None) == 'ClassVar'

    def is_collection(self, value) -> bool:
        """是否集合类型的"""
        if value is None:
            return False
        return issubclass_safe(get_origin(value), Collection)

    def is_deque(self, value) -> bool:
        """是否双端队列类型的"""
        if value is None:
            return False
        return isinstance_safe(value, collections.deque)

    def is_iterable(self, value) -> bool:
        """是否可迭代类型的"""
        if value is None:
            return False
        return value is Iterable or issubclass_safe(get_origin(value), Iterable)

    def is_list(self, value) -> bool:
        """是否列表类型的"""
        if value is None:
            return False
        return isinstance_safe(value, list) or issubclass_safe(get_origin(value), List)

    def is_tuple(self, value) -> bool:
        """是否元组类型的"""
        if value is None:
            return False
        return isinstance_safe(value, tuple) or issubclass_safe(get_origin(value), Tuple)

    def is_set(self, value) -> bool:
        """是否集合类型的"""
        if value is None:
            return False
        return isinstance_safe(value, set) or issubclass_safe(get_origin(value), Set)

    def is_mapping(self, value) -> bool:
        """是否映射类型的"""
        if value is None:
            return False
        return issubclass_safe(get_origin(value), Mapping)

    def is_dict(self, value) -> bool:
        """是否字典类型的"""
        if value is None:
            return False
        return value is Dict or isinstance_safe(value, dict) or issubclass_safe(get_origin(value), Dict)

    def is_sequence(self, value) -> bool:
        """是否序列类型的"""
        if value is None:
            return False
        return issubclass_safe(get_origin(value), Sequence)

    def is_counter(self, value) -> bool:
        """是否计数器类型的"""
        if value is None:
            return False
        return issubclass_safe(get_origin(value), Counter)

    def is_function(self, value) -> bool:
        """是否为函数"""
        return isinstance_safe(value, FunctionType)

    def is_fast_dataclass(obj):
        cls = obj if isinstance(obj, type) else type(obj)
        return hasattr(cls, _DATACLASS_FIELDS_NAME)

    def is_pydantic_model(self, value) -> bool:
        """是否为pydantic模型"""
        return hasattr(value, 'model_fields') and hasattr(value, '__pydantic_validator__')

    def get_origin_safe(self, v):
        return get_origin(v)

    def repair_type(self, value):
        """检查类型是否合法，并修正"""
        if value in self.__required_subtype_types__:
            raise RuntimeError(f"{value} 必须包含子类型")
        elif value in self.__hope_subtype_types__:
            return get_origin(self.__hope_subtype_types__[self.__hope_subtype_types__.index(value)])
        return value


"""类型解析器实例（减少类开支）"""
type_parser = TypeParser()


# -*- coding:utf-8 -*-
import re
import types
from types import MappingProxyType
from typing import TypeVar

"""用于类型标注 Used for type annotation"""
_T = TypeVar('_T')


class DefaultFactory:
    """# A sentinel object for default values to signal that a default
    factory will be used.  This is given a nice repr() which will appear
    in the function signature of dataclasses constructors."""

    def __repr__(self):
        return '<factory>'


_DEFAULT_FACTORY = DefaultFactory()


# class Unset:
#     """A sentinel object to detect if a parameter is supplied or not.  Use a class to give it a better repr."""
#
#     def __repr__(self):
#         return 'UNSET'
#
#
# UNSET = Unset()

_EMPTY_METADATA: MappingProxyType = types.MappingProxyType({})


class _PseudoField:
    """Markers for the various kinds of fields and pseudo-fields. 各种字段和伪字段的标记。"""

    def __init__(self, name):
        self.name = name

    def __repr__(self):
        return self.name


_BASE_FIELD = _PseudoField('_FIELD')
_FIELD_CLASS_VAR = _PseudoField('_FIELD_CLASS_VAR')
_FIELD_INIT_VAR = _PseudoField('_FIELD_INIT_VAR')


# The name of an attribute on the class where we store the Field
# objects.  Also used to check if a class is a Data Class.
_DATACLASS_FIELDS_NAME = 'dataclass_fields'


# The name of an attribute on the class that stores the config to
# @dataclass.
_DATACLASS_CONFIG_NAME = 'dataclass_config'

_FAST_SERIALIZER_NAME = '__fast_serializer__'

_VALIDATOR_KWARGS_NAME = 'validator_kwargs'

_SUB_VALIDATOR_KWARGS_NAME = 'sub_validator_kwargs'

_SERIALIZER_KWARGS_NAME = 'serializer_kwargs'

# 数据类装饰器名
_FAST_DATACLASS_DECORATORS_NAME = '__fast_dataclass_decorators__'

# The name of the function, that if it exists, is called at the end of
# __init__.
_POST_INIT_NAME = '__post_init__'


# String regex that string annotations for ClassVar or InitVar must match.
# https://bugs.python.org/issue33453 for details.
_MODULE_IDENTIFIER_RE = re.compile(r'^(?:\s*(\w+)\s*\.)?\s*(\w+)')


class _InitVarMeta(type):

    def __getitem__(self, params):
        return self


class InitVar(metaclass=_InitVarMeta):
    pass


# -*- coding:utf-8 -*-
from typing import Type, Dict
from .field import Field
from .constants import _DATACLASS_FIELDS_NAME, _T
from .fast_dataclass import generate_fast_dataclass


def getter(cls: Type[_T]) -> Type[_T]:

    def wrap(_cls: Type[_T]):
        return generate_getter(_cls)

    # We're called as @dataclass without parens.
    return wrap(cls)


def generate_getter_func(field_name):
    """Generate getter function for class. 为类生成getter函数"""
    def getter_func_wrapper(self):
        return getattr(self, field_name)
    return getter_func_wrapper


def generate_getter(cls: Type[_T]) -> Type[_T]:
    """Generate getter for class. 为类生成getter"""

    """需要先生成快速数据类"""
    if not hasattr(cls, _DATACLASS_FIELDS_NAME):
        cls: Type[_T] = generate_fast_dataclass(cls)  # type: ignore

    dataclass_fields: Dict[str, Field] = getattr(cls, _DATACLASS_FIELDS_NAME, {})
    for field_name, field in dataclass_fields.items():
        setattr(cls, f'get_{field_name}', generate_getter_func(field_name))
    return cls


# -*- coding:utf-8 -*-
import dataclasses
from typing import Literal
from .types import optional
from .globals import GlobalSetting


ExtraValues = Literal['allow', 'ignore', 'forbid']


@dataclasses.dataclass
class DataclassConfig:
    """快速数据类配置"""

    """数据类标题"""
    title: str = None

    """数据类描述"""
    description: str = None

    """键名驼峰转换到下划线。默认为“False”。"""
    camel_to_snake: bool = False

    """键名下划线转换到驼峰。默认为“False”。"""
    snake_to_camel: bool = False

    """是否为str类型去掉前导和尾部空白。"""
    str_strip_whitespace: bool = False

    """str类型的最小长度，空为不限制"""
    str_min_length: optional[int] = None

    """str类型的最大长度，空为不限制"""
    str_max_length: optional[int] = None

    """
    *`allow`-允许任何额外的属性。
    *`forbid`-禁止任何额外的属性。
    *`ignore`-忽略任何额外的属性。
    """
    extra: optional[ExtraValues] = 'ignore'

    """数据类是否是伪不可变的"""
    frozen: bool = False

    """
    数据类更改时是否验证数据。默认为“False”。
    FastDataclass的默认行为是在创建数据类时验证数据。
    如果用户在创建数据类后更改了数据，则不重新验证数据类。"""
    validate_on_change: bool = False

    """是否允许无穷大（“+inf”和“-inf”）和NaN值浮动字段。默认为“True”。"""
    allow_inf_nan: bool = True

    """datetime格式化"""
    datetime_format: str = "%Y-%m-%d %H:%M:%S"

    """date格式化"""
    date_format: str = "%Y-%m-%d"

    """默认必填"""
    required: optional[bool] = None

    eq: bool = True

    order: bool = False

    unsafe_hash: bool = False

    def __post_init__(self):
        if self.required is None:
            self.required = GlobalSetting.get_dataclass_default_required()


# -*- coding:utf-8 -*-
from typing import Union, Any, Callable, Optional, Dict, Type
from .field import Field
from dataclasses import dataclass, field as dataclass_field


@dataclass
class Decorator:
    """基础的装饰器信息"""

    clazz: str
    class_name: str
    func: Callable[..., Any]


@dataclass
class FieldValidatorDecoratorInfo(Decorator):
    """自定义字段验证装饰器"""

    fields: tuple[str, ...]


@dataclass
class FastDataclassDecoratorInfo:
    """快速数据类装饰器信息"""

    field_validators: Dict[str, FieldValidatorDecoratorInfo] = dataclass_field(default_factory=dict)

    @classmethod
    def build(cls, fast_dataclass: Type[Any]):
        decorator_info = cls()

        return decorator_info


def field_serializer(*fields: Union[str, Field]):
    """自定义字段序列化"""
    pass


def field_validator(field: Union[str, Field], *fields: Union[str, Field]):
    """自定义字段验证"""
    fields = field, *fields
    fields = [field.name if isinstance(field, Field) else field for field in fields]
    if not all(isinstance(field, str) for field in fields):
        raise RuntimeError(
            '`@field_validator` 字段应作为单独的字符串或字段参数传递。 '
            "E.g. usage should be `@validator('<field_name_1>', '<field_name_2>', ...)`"
        )


def type_serializer(_type: Any):
    """自定义类型序列化"""
    pass


def type_validator(_type: Any):
    """自定义类型验证"""
    pass


def dataclass_serializer(func: Optional[Callable[..., Any]] = None):
    """自定义数据类序列化"""
    pass


def dataclass_validator(func: Optional[Callable[..., Any]] = None):
    """自定义数据类序列化"""
    pass


# -*- coding:utf-8 -*-


class GlobalSetting:
    """全局设置"""

    """语言代码 Language code"""
    LANGUAGE_CODE = 'zh-Hans'

    """数据类默认必填"""
    DATACLASS_DEFAULT_REQUIRED = False

    """是否启用国际化"""
    USE_I18N = True

    @classmethod
    def get_language(cls):
        return cls.LANGUAGE_CODE
    
    @classmethod
    def set_language(cls, language_code):
        cls.LANGUAGE_CODE = language_code
        return cls.LANGUAGE_CODE

    @classmethod
    def get_i18n(cls):
        return cls.USE_I18N

    @classmethod
    def set_i18n(cls, use_i18n):
        cls.USE_I18N = use_i18n
        return cls.USE_I18N

    @classmethod
    def get_dataclass_default_required(cls):
        return cls.DATACLASS_DEFAULT_REQUIRED

    @classmethod
    def set_dataclass_default_required(cls, default_required: bool):
        cls.DATACLASS_DEFAULT_REQUIRED = default_required
        return cls.DATACLASS_DEFAULT_REQUIRED


# -*- coding:utf-8 -*-
import dataclasses
from typing import Optional, Any, Callable, Union
from .types import optional, number
from .utils import _recursive_repr, _format_type
from .validator import Validator


class Field:
    """字段"""

    """字段名"""
    name: str

    """JsonSchema标题"""
    title: optional[str]

    """注解"""
    annotation: optional[type[Any]]

    """验证器"""
    validator: optional[Validator]

    """默认值"""
    default: Any

    """默认工厂"""
    default_factory: optional[Callable]

    """是否必填"""
    required: optional[bool]

    """是否初始化"""
    init: optional[bool]

    """是否显示"""
    repr: optional[bool]

    """别名"""
    alias: optional[str]

    """验证别名"""
    val_alias: optional[str]

    """序列化别名"""
    ser_alias: optional[str]

    """最小"""
    min: optional[number]

    """最大"""
    max: optional[number]

    """最短"""
    min_length: optional[int]

    """最长"""
    max_length: optional[int]

    """描述"""
    description: optional[str]

    """排除序列化"""
    exclude: optional[bool]

    """是否抛弃"""
    deprecated: optional[bool]

    """是否冻结"""
    frozen: optional[bool]

    """"""
    init_var: optional[bool]

    """验证器参数"""
    validator_kwargs: optional[Union[dict, list]]

    """序列化器参数"""
    ser_extras: optional[Union[dict, list]]

    """子验证器参数"""
    sub_validator_kwargs: optional[Union[dict, list]]

    """子序列化器参数"""
    subser_extras: optional[Union[dict, list]]

    __slots__ = (
        'name',
        'title',
        'annotation',
        'validator',
        'default',
        'default_factory',
        'required',
        'init',
        'repr',
        'alias',
        'val_alias',
        'ser_alias',
        'min',
        'max',
        'min_length',
        'max_length',
        'description',
        'exclude',
        'deprecated',
        'frozen',
        'init_var',
        'validator_kwargs',
        'serializer_kwargs',
        'sub_validator_kwargs',
        'sub_serializer_kwargs',
        '_field_type'
    )

    def __init__(self, **kwargs):
        kwargs = {k: kwargs.get(k, _DEFAULT_FIELD_VALUES.get(k)) for k in self.__slots__}
        _ = {setattr(self, k, v) for k, v in kwargs.items()}

        if self.default is dataclasses.MISSING:
            self.default = None

        if self.default is not None and self.default_factory is not None:
            raise ValueError(f'不能同时指定 default 和 default_factory')

    def set_annotation(self, annotation):
        if annotation is Optional:
            raise RuntimeError("Optional 必须添加内部类型")
        self.annotation = annotation

    @_recursive_repr
    def __repr__(self):
        return (f"Field(name={self.name!r}, "
                f"annotation={_format_type(self.annotation)}, "
                f"default={self.default!r}, "
                f"default_factory={self.default_factory!r}, "
                f"required={self.required!r}, "
                f"init={self.init!r}, "
                f"repr={self.repr!r}, "
                f"description={self.description!r})")

    def get_default_value(self):
        value = None if self.default is None else self.default
        if value is None:
            value = self.default_factory() if self.default_factory is not None else value
        return value


# This function is used instead of exposing Field creation directly,
# so that a type checker can be told (via overloads) that this is a
# function whose type depends on its parameters.
def field(
    default=None,
    *,
    default_factory=None,
    required: optional[bool] = None,
    min_length: optional[int] = None,
    max_length: optional[int] = None,
    validator_kwargs: optional[dict] = None,
    sub_validator_kwargs: optional[Union[dict, list]] = None,
    **kwargs
) -> Field:
    """Return an object to identify dataclass fields.

    default is the default value of the field.  default_factory is a
    0-argument function called to initialize a field's value.  If init
    is True, the field will be a parameter to the class's __init__()
    function.  If repr is True, the field will be included in the
    object's repr().  If hash is True, the field will be included in
    the object's hash().  If compare is True, the field will be used
    in comparison functions.  metadata, if specified, must be a
    mapping which is stored but not otherwise examined by dataclass.

    It is an error to specify both default and default_factory.
    """
    return Field(default=default, default_factory=default_factory, required=required, min_length=min_length,
                 max_length=max_length, validator_kwargs=validator_kwargs, sub_validator_kwargs=sub_validator_kwargs,
                 **kwargs)


_DEFAULT_FIELD_VALUES: dict = dict(
    name=None,
    title=None,
    vaildator=None,
    annotation=None,
    default=None,
    default_factory=None,
    required=None,
    init=True,
    repr=True,
    alias=None,
    val_alias=None,
    ser_alias=None,
    min=None,
    max=None,
    min_length=None,
    max_length=None,
    description=None,
    exclude=None,
    deprecated=None,
    frozen=None,
    init_var=None,
    validator_kwargs=None,
    ser_extras=None,
    sub_validator_kwargs=None,
    subser_extras=None,
)


# -*- coding:utf-8 -*-
import enum
from typing import Type, Dict, Any, Union, Tuple
from .constants import _T, _DATACLASS_FIELDS_NAME, _POST_INIT_NAME
from .exceptions import ErrorDetail, ValidationError
from .field import Field
from .type_parser import type_parser
from .utils import _format_type
from .validator import Validator, EnumValidator, TupleValidator
from .exceptions import DataclassCustomError


class FastSerializer:
    """序列化器提供强大的序列化和反序列化"""

    catch_val_exceptions: Tuple[Type[Exception]] = (ValueError, TypeError, RuntimeError, DataclassCustomError)

    def __init__(self):
        self.field_validators: Dict[str, Validator] = {}  # 自定义的字段验证器
        # self.type_validators: Dict[Any, Validator] = {}  # 用户编写的自定义类型验证器

    def deserialize(self, data: Union[dict, object], target=None, instance=None) -> _T:
        """反序列化 Deserialize"""
        # if target is None and instance is None:
        #     raise RuntimeError('目标类和实例不能同时为空')

        if target is None:
            target = instance.__class__
        class_name: str = target.__name__
        dataclass_fields: Dict[str, Field] = getattr(target, _DATACLASS_FIELDS_NAME, {})
        errors = []
        for field_name, field in dataclass_fields.items():
            input_value = self._get_value(data, True, field_name, field)
            try:
                """数据为必填项"""
                if input_value is None and field.required:
                    raise DataclassCustomError('missing', f"{field_name} 为必填项")
                elif input_value is not None:
                    input_value = field.validator.validate(input_value)

                setattr(instance, field_name, input_value)
            except ValidationError as e:
                for error in e.line_errors:
                    error.loc.insert(0, field_name)
                errors.extend(e.line_errors)
            except self.catch_val_exceptions as e:
                # raise e
                if type(e) is DataclassCustomError:
                    error_detail = ErrorDetail(
                        # key=field_name,
                        loc=[field_name],
                        input_value=input_value,
                        exception_type=e.exception_type,
                        msg=e.msg
                    )
                else:
                    error_detail = ErrorDetail(
                        # key=field_name,
                        loc=[field_name],
                        input_value=input_value,
                        exception_type=e,
                        msg=str(e)
                    )
                errors.append(error_detail)

        if errors:
            raise ValidationError(title=class_name, line_errors=errors)

        # post_init
        if hasattr(instance, _POST_INIT_NAME):
            getattr(instance, _POST_INIT_NAME)()

        return instance  # type: ignore

    @staticmethod
    def _get_value(data: Union[dict, object], is_dict: bool, field_name: str, field: Field) -> Any:
        try:
            value = data.__getitem__(field_name) if is_dict else getattr(data, field_name)
        except (KeyError, AttributeError):
            value = field.get_default_value()
        return value


# -*- coding:utf-8 -*-
import _thread
import builtins
import functools
import sys
from dataclasses import Field as DataclassField
from types import MemberDescriptorType, GenericAlias, FunctionType
from typing import ClassVar, Dict, Type, Any, List

from .constants import (_T, _DATACLASS_CONFIG_NAME, _DATACLASS_FIELDS_NAME, _BASE_FIELD,
                        _MODULE_IDENTIFIER_RE, InitVar, _FIELD_CLASS_VAR, _FIELD_INIT_VAR,
                        _FAST_DATACLASS_DECORATORS_NAME,
                        _FAST_SERIALIZER_NAME, _SUB_VALIDATOR_KWARGS_NAME)
from .dataclass_config import DataclassConfig
from .decorators import FastDataclassDecoratorInfo
from .field import Field
from .serializer import FastSerializer
from .validator import matching_validator


def _tuple_str(obj_name, fields: List[Field]):
    # Return a string representing each field of obj_name as a tuple
    # member.  So, if fields is ['x', 'y'] and obj_name is "self",
    # return "(self.x,self.y)".

    # Special case for the 0-tuple.
    if not fields:
        return '()'
    # Note the trailing comma, needed if this turns out to be a 1-tuple.
    return f'({",".join([f"{obj_name}.{f.name}" for f in fields])},)'


# This function's logic is copied from "recursive_repr" function in
# repr lib module to avoid dependency.
def _recursive_repr(user_function):
    """Decorator to make a repr function return "..." for a recursive call."""
    repr_running = set()

    @functools.wraps(user_function)
    def wrapper(self):
        key = id(self), _thread.get_ident()
        if key in repr_running:
            return '...'
        repr_running.add(key)
        try:
            result = user_function(self)
        finally:
            repr_running.discard(key)
        return result

    return wrapper


def _create_fn(name, args, body, *, _globals=None, _locals=None, return_type=None):
    # Note that we mutate locals when exec() is called.  Caller
    # beware!  The only callers are internal to this module, so no
    # worries about external callers.
    if _locals is None:
        _locals = {}
    if 'BUILTINS' not in _locals:
        _locals['BUILTINS'] = builtins
    return_annotation = ''
    if return_type is not None:
        _locals['_return_type'] = return_type
        return_annotation = '->_return_type'
    args = ','.join(args)
    body = '\n'.join(f'  {b}' for b in body)

    # Compute the text of the entire function.
    txt = f' def {name}({args}){return_annotation}:\n{body}'

    local_vars = ', '.join(_locals.keys())
    txt = f"def __create_fn__({local_vars}):\n{txt}\n return {name}"

    ns = {}
    exec(txt, _globals, ns)
    return ns['__create_fn__'](**_locals)


def _field_assign(frozen, name, value, self_name):
    # If we're a frozen class, then assign to our fields in __init__
    # via object.__setattr__.  Otherwise, just use a simple
    # assignment.
    #
    # self_name is what "self" is called in this function: don't
    # hard-code "self", since that might be a field name.
    if frozen:
        return f'BUILTINS.object.__setattr__({self_name},{name!r},{value})'
    return f'{self_name}.{name}={value}'


def _repr_fn(fields, _globals):
    fn = _create_fn('__repr__',
                    ('self',),
                    ['return self.__class__.__qualname__ + f"(' +
                     ', '.join([f"{f.name}={{self.{f.name}!r}}"
                                for f in fields]) +
                     ')"'],
                    _globals=_globals)
    return _recursive_repr(fn)


def _hash_fn(fields, _globals):
    self_tuple = _tuple_str('self', fields)
    return _create_fn('__hash__',
                      ('self',),
                      [f'return hash({self_tuple})'],
                      _globals=_globals)


def _is_class_var(field_type: Any):
    # This test uses a typing internal class, but it's the best way to
    # test if this is a ClassVar.
    return field_type is ClassVar or (type(field_type) is GenericAlias and field_type.__origin__ is ClassVar)


def _is_init_var(field_type: Any):
    # The module we're checking against is the module we're
    # currently in (dataclasses_plus.py).
    return field_type is InitVar


def _is_type(annotation, cls, a_module, a_type, is_type_predicate):
    # Given a type annotation string, does it refer to a_type in
    # a_module?  For example, when checking that annotation denotes a
    # ClassVar, then a_module is typing, and a_type is
    # typing.ClassVar.

    # It's possible to look up a_module given a_type, but it involves
    # looking in sys.modules (again!), and seems like a waste since
    # the caller already knows a_module.

    # - annotation is a string type annotation
    # - cls is the class that this annotation was found in
    # - a_module is the module we want to match
    # - a_type is the type in that module we want to match
    # - is_type_predicate is a function called with (obj, a_module)
    #   that determines if obj is of the desired type.

    # Since this test does not do a local namespace lookup (and
    # instead only a module (global) lookup), there are some things it
    # gets wrong.

    # With string annotations, cv0 will be detected as a ClassVar:
    #   CV = ClassVar
    #   @dataclass
    #   class C0:
    #     cv0: CV

    # But in this example cv1 will not be detected as a ClassVar:
    #   @dataclass
    #   class C1:
    #     CV = ClassVar
    #     cv1: CV

    # In C1, the code in this function (_is_type) will look up "CV" in
    # the module and not find it, so it will not consider cv1 as a
    # ClassVar.  This is a fairly obscure corner case, and the best
    # way to fix it would be to eval() the string "CV" with the
    # correct global and local namespaces.  However, that would involve
    # a eval() penalty for every single field of every dataclass
    # that's defined.  It was judged not worth it.

    match = _MODULE_IDENTIFIER_RE.match(annotation)
    if match:
        ns = None
        module_name = match.group(1)
        if not module_name:
            # No module name, assume the class's module did
            # "from dataclasses import InitVar".
            ns = sys.modules.get(cls.__module__).__dict__
        else:
            # Look up module_name in the class's module.
            module = sys.modules.get(cls.__module__)
            if module and module.__dict__.get(module_name) is a_module:
                ns = sys.modules.get(a_type.__module__).__dict__
        if ns and is_type_predicate(ns.get(match.group(2)), a_module):
            return True
    return False


def _generate_field(cls, field_name: str, annotation: type, dataclass_config: DataclassConfig) -> Field:
    # Return a Field object for this field name and type.  ClassVars
    # and InitVars are also returned, but marked as such (see f._field_type).
    # 返回此字段名称和类型的Field对象。ClassVars和InitVars也会返回，但标记为这样（请参阅f.field_type）。

    # If the default value isn't derived from Field, then it's only a
    # normal default value.  Convert it to a Field().
    # 如果默认值不是从Field派生的，则它只是一个普通的默认值。将其转换为Field()。

    field_or_default = getattr(cls, field_name, None)
    if isinstance(field_or_default, Field):
        """已是Field对象"""
        field = field_or_default
    elif isinstance(field_or_default, DataclassField):
        """原始数据类字段"""
        field = Field(default=field_or_default.default, default_factory=field_or_default.default_factory,
                      init=field_or_default.init, repr=field_or_default.repr)
    else:
        if isinstance(field_or_default, MemberDescriptorType):
            # This is a field in __slots__, so it has no default value.
            # 这是__slots__中的字段，因此它没有默认值。
            field_or_default = None
        field = Field(default=field_or_default)
        setattr(cls, field_name, field)  # 保留Field在未实例的数据类上

    field.name = field_name
    field.required = dataclass_config.required
    field.frozen = dataclass_config.frozen
    field.set_annotation(annotation)

    # 组装验证器参数传递
    field.validator_kwargs = field.validator_kwargs or dict()
    if field.min_length:
        field.validator_kwargs['min_length'] = field.min_length
    if field.max_length:
        field.validator_kwargs['max_length'] = field.max_length
    if field.sub_validator_kwargs:
        field.validator_kwargs[_SUB_VALIDATOR_KWARGS_NAME] = field.sub_validator_kwargs
    # 查找对应类型的验证器
    field.validator = matching_validator(annotation, **field.validator_kwargs)

    # 接下来是对InitVar和ClassVar的支持
    # Assume it's a normal field until proven otherwise.  We're next
    # going to decide if it's a ClassVar or InitVar, everything else
    # is just a normal field.
    # 在证明之前，假设它是一个普通字段。接下来，我们将决定它是ClassVar还是InitVar，其他所有内容都只是普通字段。
    setattr(field, '_field_type', _BASE_FIELD)

    # In addition to checking for actual types here, also check for
    # string annotations.  get_type_hints() won't always work for us
    # (see https://github.com/python/typing/issues/508 for example),
    # plus it's expensive and would require an eval for every string
    # annotation.  So, make the best effort to see if this is a ClassVar
    # or InitVar using regex's and checking that the thing referenced
    # is actually of the correct type.

    # For the complete discussion, see https://bugs.python.org/issue33453

    # If typing has not been imported, then it's impossible for any
    # annotation to be a ClassVar.  So, only look for ClassVar if
    # typing has been imported by any module (not necessarily cls module).
    _typing = sys.modules.get('typing')
    if _typing:
        end_if = isinstance(field.annotation, str) and _is_type(field.annotation, cls, _typing, ClassVar, _is_class_var)
        if _is_class_var(annotation) or end_if:
            annotation._field_type = _FIELD_CLASS_VAR

    # If the type is InitVar, or if it's a matching string annotation, then it's an InitVar.
    field_type = getattr(annotation, '_field_type', _BASE_FIELD)
    if field_type is _BASE_FIELD:
        # The module we're checking against is the module we're currently in (dataclasses_plus.py).
        module = sys.modules[__name__]
        if (_is_init_var(field.annotation) or (isinstance(field.annotation, str)
                                               and _is_type(field.annotation, cls, module, InitVar, _is_init_var))):
            setattr(field, '_field_type', _FIELD_INIT_VAR)

    # Validations for individual fields.  This is delayed until now,
    # instead of in the Field() constructor, since only here do we
    # know the field name, which allows for better error reporting.

    # Special restrictions for ClassVar and InitVar.
    if field_type in (_FIELD_CLASS_VAR, _FIELD_INIT_VAR):
        if field.default_factory is not None:
            raise TypeError(f'field {field.name} cannot have a default factory')
        # Should I check for other field settings? default_factory
        # seems the most serious to check for.  Maybe add others.  For
        # example, how about init=False (or really,
        # init=<not-the-default-init-value>)?  It makes no sense for
        # ClassVar and InitVar to specify init=<anything>.

    # For real fields, disallow mutable defaults for known types.
    if field_type is _BASE_FIELD and isinstance(field.default, (list, dict, set)):
        raise ValueError(f'mutable default {type(field.default)} for field {field.name} is not allowed: use default_factory')

    return field


def _set_new_attribute(cls, name, value):
    # Never overwrites an existing attribute.  Returns True if the
    # attribute already exists.
    if name in cls.__dict__:
        return True
    setattr(cls, name, value)
    return False


# Decide if/how we're going to create a hash function.  Key is
# (unsafe_hash, eq, frozen, does-hash-exist).  Value is the action to
# take.  The common case is to do nothing, so instead of providing a
# function that is a no-op, use None to signify that.
def _hash_set_none(_, __, _globals):
    return None


def _hash_add(cls, fields, _globals):
    fields = [f for f in fields if (f.compare if f.hash is None else f.hash)]
    return _set_qualname(cls, _hash_fn(fields, _globals))


def _hash_exception(cls, _, _globals):
    # Raise an exception.
    raise TypeError(f'Cannot overwrite attribute __hash__ in class {cls.__name__}')


#
#                +-------------------------------------- unsafe_hash?
#                |      +------------------------------- eq?
#                |      |      +------------------------ frozen?
#                |      |      |      +----------------  has-explicit-hash?
#                |      |      |      |
#                |      |      |      |        +-------  action
#                |      |      |      |        |
#                v      v      v      v        v
_hash_action = {(False, False, False, False): None,
                (False, False, False, True): None,
                (False, False, True, False): None,
                (False, False, True, True): None,
                (False, True, False, False): _hash_set_none,
                (False, True, False, True): None,
                (False, True, True, False): _hash_add,
                (False, True, True, True): None,
                (True, False, False, False): _hash_add,
                (True, False, False, True): _hash_exception,
                (True, False, True, False): _hash_add,
                (True, False, True, True): _hash_exception,
                (True, True, False, False): _hash_add,
                (True, True, False, True): _hash_exception,
                (True, True, True, False): _hash_add,
                (True, True, True, True): _hash_exception,
                }


# See https://bugs.python.org/issue32929#msg312829 for an if-statement
# version of this table.


def _set_qualname(cls, value):
    # Ensure that the functions returned from _create_fn uses the proper
    # __qualname__ (the class they belong to).
    if isinstance(value, FunctionType):
        value.__qualname__ = f"{cls.__qualname__}.{value.__name__}"
    return value


def generate_fast_dataclass(cls: Type[_T]) -> Type[_T]:
    """生成快速数据类"""

    # Now that dicts retain insertion order, there's no reason to use
    # an ordered dict.  I am leveraging that ordering here, because
    # derived class fields overwrite base class fields, but the order
    # is defined by the base class, which is found first.
    # 既然dicts保留了插入顺序，就没有理由使用一个有序的格言。
    # 我在这里利用这个有序，因为派生类字段覆盖基类字段，但顺序由基类定义，该基类首先找到。
    dataclass_fields = {}

    if cls.__module__ in sys.modules:
        _globals = sys.modules[cls.__module__].__dict__
    else:
        # Theoretically this can happen if someone writes
        # a custom string to cls.__module__.  In which case
        # such dataclass won't be fully introspectable
        # (w.r.t. typing.get_type_hints) but will still function
        # correctly.
        _globals = {}

    # 设置快速数据类的配置
    dataclass_config = getattr(cls, _DATACLASS_CONFIG_NAME, None)
    if dataclass_config is None:
        dataclass_config = DataclassConfig()
        setattr(cls, _DATACLASS_CONFIG_NAME, dataclass_config)

    # 设置快速数据类的序列化器
    fast_serializer = getattr(cls, _FAST_SERIALIZER_NAME, None)
    if fast_serializer is None:
        fast_serializer = FastSerializer()
        setattr(cls, _FAST_SERIALIZER_NAME, fast_serializer)

    # Find our base classes in reverse MRO order, and exclude
    # ourselves.  In reversed order so that more derived classes
    # override earlier field definitions in base classes.  As long as
    # we're iterating over them, see if any are frozen.
    # 以相反的MRO顺序查找我们的基类，并排除我们自己以相反的顺序，使更多的派生类重写基类中早期的字段定义。
    # 只要我们正在对它们进行迭代，看看是否有冻结的。

    # any_frozen_base = False
    # has_dataclass_bases = False
    for base in cls.__mro__[-1:0:-1]:
        # Only process classes that have been processed by our
        # decorator.  That is, they have a _FIELDS attribute.
        base_fields = getattr(base, _DATACLASS_FIELDS_NAME, None)
        if base_fields:
            has_dataclass_bases = True
            for _field in base_fields.values():
                dataclass_fields[_field.name] = _field
                cls.__annotations__[_field.name] = _field.type
                setattr(cls, _field.name, _field)  # 让未实例时获取为字段信息
            # if getattr(base, _DATACLASS_CONFIG_NAME).frozen:
            #     any_frozen_base = True

    # Annotations that are defined in this class (not in base
    # classes).  If __annotations__ isn't present, then this class
    # adds no new annotations.  We use this to compute fields that are
    # added by this class.
    #
    # Fields are found from cls_annotations, which is guaranteed to be
    # ordered.  Default values are from class attributes, if a field
    # has a default.  If the default value is a Field(), then it
    # contains additional info beyond (and possibly including) the
    # actual default value.  Pseudo-fields ClassVars and InitVars are
    # included, despite the fact that they're not real fields.  That's
    # dealt with later.
    # 此类中定义的注释（不在基类中类）。如果__annotations__不存在，则此类不添加新的注释。我们使用它来计算由此类添加。
    # 字段是从cls_annotations中找到的，保证为命令。默认值来自类属性，如果字段具有默认值。
    # 如果默认值是Field（），则它包含超出（可能包括）的其他信息实际默认值。伪字段ClassVars和InitVars是 包括在内，尽管它们不是真正的领域。
    # 那是稍后处理。
    cls_annotations = cls.__dict__.get('__annotations__', {})

    # Now find fields in our class.  While doing so, validate some
    # things, and set the default values (as class attributes) where we can.
    # 现在在我们班上查找字段。在这样做的同时，验证一些并设置默认值（作为类属性），其中我们可以。
    cls_fields = [_generate_field(cls, name, annotation, dataclass_config) for name, annotation in cls_annotations.items()]
    [dataclass_fields.__setitem__(_field.name, _field) for _field in cls_fields]  # type: ignore

    # Do we have any Field members that don't also have annotations?
    # 我们有没有字段成员也没有注释？
    for field_name, value in cls.__dict__.items():
        if isinstance(value, Field) and field_name not in cls_annotations:
            raise TypeError(f'{field_name!r} is a field but has no type annotation')

    # Remember all the fields on our class (including bases).  This
    # also marks this class as being a dataclass.
    # 记住我们班上的所有字段（包括基数）。这还将该类标记为数据类。
    setattr(cls, _DATACLASS_FIELDS_NAME, dataclass_fields)

    # 设置装饰器们
    dataclass_decorators = FastDataclassDecoratorInfo.build(cls)
    setattr(cls, _FAST_DATACLASS_DECORATORS_NAME, dataclass_decorators)

    # Was this class defined with an explicit __hash__?  Note that if
    # __eq__ is defined in this class, then python will automatically
    # set __hash__ to None.  This is a heuristic, as it's possible
    # that such a __hash__ == None was not auto-generated, but it
    # closes enough.
    # class_hash = cls.__dict__.get('__hash__', None)
    # has_explicit_hash = not (class_hash is None or (class_hash is None and '__eq__' in cls.__dict__))

    # If we're generating ordering methods, we must be generating the
    # eq methods.
    if dataclass_config.order and not dataclass_config.eq:
        raise ValueError('eq must be true if order is true')

    # Get the fields as a list, and include only real fields.  This is
    # used in all the following methods.
    # field_list = [f for f in dataclass_fields.values() if getattr(f, '_field_type', None) is _BASE_FIELD]

    # if dataclass_config.repr:
    #     _fields = [f for f in field_list if f.repr]
    #     _set_new_attribute(cls, '__repr__', _repr_fn(_fields, globals))

    # Decide if/how we're going to create a hash function.
    # hash_action = _hash_action[bool(dataclass_config.unsafe_hash), bool(dataclass_config.eq), bool(dataclass_config.frozen),
    #                            has_explicit_hash]
    # if hash_action:
    #     # No need to call _set_new_attribute here, since by the time
    #     # we're here the overwriting is unconditional.
    #     cls.__hash__ = hash_action(cls, field_list, _globals)
    #
    # if not getattr(cls, '__doc__'):
    #     # Create a class doc-string.
    #     try:
    #         # In some cases fetching a signature is not possible.
    #         # But, we surely should not fail in this case.
    #         text_sig = str(inspect.signature(cls)).replace(' -> None', '')
    #     except (TypeError, ValueError):
    #         text_sig = ''
    #     cls.__doc__ = (cls.__name__ + text_sig)

    return cls


# @dataclass_transform(kw_only_default=True, field_specifiers=(Field,))
class FastDataclassMeta(type):

    def __new__(cls, name, bases, dct, **kwargs):
        if not bases:
            # This is the FastDataclass class itself being created and does not require any logic
            # 这是正在创建的快速数据类本身，不需要任何逻辑
            return super().__new__(cls, name, bases, dct, **kwargs)

        _cls: Type['FastDataclass'] = super().__new__(cls, name, bases, dct, **kwargs)  # type: ignore

        return generate_fast_dataclass(_cls)


class FastDataclass(metaclass=FastDataclassMeta):
    """Base class for creating fast dataclass 用于创建快速数据类的基类"""

    """数据类配置"""
    dataclass_config: ClassVar[DataclassConfig]

    """在数据类上定义的字段的元数据"""
    dataclass_fields: ClassVar[Dict[str, Field]]

    """用于存放数据类的装饰器数据"""
    # __fast_dataclass_decorators__: ClassVar[FastDataclassDecoratorInfo]

    """JsonSchema"""
    # __fast_schema__: ClassVar[JsonSchema] = JsonSchema()

    """快速序列化器，支持反序列化和序列化"""
    __fast_serializer__: ClassVar[FastSerializer]

    def __init__(self, /, **kwargs):
        # 隐藏特定代码的回溯信息，以便在发生异常时，使回溯信息更加简洁和有意义。
        __tracebackhide__ = True
        self.__fast_serializer__.deserialize(kwargs, instance=self)

    def __post_init__(self):
        """
        Override this method to perform additional initialization after `__init__``.
        重写此方法以在`__init之后执行附加初始化__`，这将非常有用。
        """
        pass

    # def schema_dict(self):
    #     raise NotImplementedError('')

    def __str__(self):
        return f"{self.__class__.__name__}({self.__repr_names__(', ')})"

    def __repr__(self) -> str:
        return self.__str__()

    def __repr_names__(self, join_str: str):
        return join_str.join(repr(v) if a is None else f'{a}={v!r}' for a, v in self.__repr_values__())

    def __repr_values__(self):
        for k, v in self.__dict__.items():
            field = self.dataclass_fields.get(k)
            if field and field.repr:
                yield k, v



# -*- coding:utf-8 -*-
import collections.abc
import datetime
import decimal
import enum
import inspect
import re
import time
import typing_extensions
import uuid
from abc import ABC, abstractmethod
from decimal import Decimal
from types import FunctionType
from typing import (
    Any, Generator, Union, Collection, Iterable, Literal, List, get_args, Tuple, Set, Dict, Optional,
    Sequence, Mapping, Callable, TypedDict, FrozenSet, Type, Deque, is_typeddict
)
from .constants import _T
from .exceptions import DataclassCustomError, ErrorDetail, ValidationError, ValidatorBuildingError
from .type_parser import type_parser
from .types import optional
from .utils import _format_type, get_sub_validator_kwargs, isinstance_safe, issubclass_safe


class Validator(ABC):
    """验证器基类"""

    validator_name: str
    annotation: _T

    def __init__(self, **kwargs): ...

    @abstractmethod
    def validate(self, value): ...

    @property
    def name(self) -> str:
        return self.validator_name

    def __repr__(self):
        return f"{self.__class__.__name__}(\n  name: {self.name!r},\n  annotation: {self.annotation}\n)"

    @classmethod
    def build(cls, annotation: _T, **kwargs) -> 'Validator':
        return cls(annotation=annotation, **kwargs)


class AnyValidator(Validator):
    """任意类型验证器"""

    validator_name = 'any'
    annotation = Any

    def validate(self, value):
        return value


class StringValidator(Validator):
    """字符串验证器"""

    validator_name = 'str'
    annotation = str
    allow_number: bool = True

    def __init__(self, allow_number: bool = True, **kwargs):
        super().__init__(**kwargs)
        self.allow_number = allow_number

    def validate(self, value, allow_number: bool = None) -> str:
        self.allow_number = self.allow_number if allow_number is None else allow_number
        maybe_str = self.maybe_str(value)
        if maybe_str:
            return maybe_str
        elif isinstance_safe(value, bytearray):
            return value.decode('utf-8')
        numbers_types = (IntegerValidator.annotation, FloatValidator.annotation,
                         DecimalValidator.annotation, BoolValidator.annotation)
        if self.allow_number and isinstance_safe(value, numbers_types):
            return self.annotation(value)
        raise ValueError('输入应为有效字符串')

    @classmethod
    def maybe_str(cls, value, raise_error: bool = True) -> optional[str]:
        if isinstance_safe(value, cls.annotation):
            return value
        elif isinstance_safe(value, BytesValidator.annotation):
            try:
                return value.decode('utf-8')
            except UnicodeDecodeError:
                if not raise_error:
                    return False
                raise DataclassCustomError('string_unicode', '输入应为有效字符串，不能将原始数据解析为unicode字符串')
        return None


class BoolValidator(Validator):
    """布尔验证器"""

    validator_name = 'bool'
    annotation = bool

    def validate(self, value) -> bool:
        if isinstance_safe(value, self.annotation):
            return value
        elif isinstance_safe(value, IntegerValidator.annotation):
            return self.int_to_bool(value)
        elif isinstance_safe(value, FloatValidator.annotation):
            return self.int_to_bool(IntegerValidator.annotation(value))
        elif isinstance_safe(value, StringValidator.annotation):
            return self.str_to_bool(value)
        raise ValueError('输入应为有效布尔类型')

    @staticmethod
    def str_to_bool(value: str) -> bool:
        if value.lower() in ['0', 'false', 'f', 'n', 'no', 'off', '不', '否', '错误', '异常', '错']:
            return False
        elif value.lower() in ['1', 'true', 't', 'y', 'yes', 'on', '是', '好', '好的', '正确', '对', '对的']:
            return True
        raise ValueError('输入应为有效布尔类型')

    @staticmethod
    def int_to_bool(value: int) -> bool:
        if value == 0:
            return False
        elif value == 1:
            return True
        raise ValueError('输入应为有效布尔类型')


class IntegerValidator(Validator):
    """整型验证器"""

    validator_name = 'int'
    annotation = int
    half_adjust_value = 0.11

    def validate(self, value):
        if isinstance_safe(value, self.annotation):
            return value
        try:
            maybe_str = StringValidator.maybe_str(value)
            if maybe_str:
                return int(value)
        except (UnicodeDecodeError, ValueError, DataclassCustomError):
            raise DataclassCustomError('int_parsing', '输入应为有效整数，无法将字符串解析为整数')
        if isinstance_safe(value, (float, Decimal)):
            return self.float_or_decimal_to_int(value)
        try:
            if issubclass_safe(value, enum.Enum):
                return self.enum_to_int(value)
        except (ValueError, TypeError):
            raise ValueError('输入应为有效整数')
        raise ValueError('输入应为有效整数')

    @staticmethod
    def bool_to_int(value: bool):
        return IntegerValidator.annotation(value)

    @staticmethod
    def str_to_int(value: str):
        return IntegerValidator.annotation(value)

    def float_or_decimal_to_int(self, value: float, is_decimal: bool = False):
        # 四舍五入容差
        integer_part = self.annotation(value)
        maybe_int = self.annotation(self.annotation(value + self.half_adjust_value if not is_decimal
                                                    else Decimal(self.half_adjust_value)))
        return maybe_int if maybe_int > integer_part else integer_part

    @staticmethod
    def enum_to_int(value: enum.Enum):
        return int(value.value)


class FloatValidator(Validator):
    """浮点验证器"""

    validator_name = 'float'
    annotation = float

    def validate(self, value):
        if isinstance_safe(value, self.annotation):
            return value
        elif isinstance_safe(value, IntegerValidator.annotation):
            return self.annotation(value)
        elif isinstance_safe(value, DecimalValidator.annotation):
            return self.annotation(value)
        try:
            maybe_str = StringValidator.maybe_str(value)
            if maybe_str:
                return self.annotation(value)
        except (UnicodeDecodeError, ValueError, TypeError, DataclassCustomError):
            raise DataclassCustomError('float_parsing', '输入应为有效数字，无法将字符串解析为数字')
        raise DataclassCustomError('float_parsing', '输入应为有效浮点值')


class DecimalValidator(FloatValidator):
    """Decimal验证器"""

    validator_name = 'decimal'
    annotation = Decimal

    def validate(self, value):
        if isinstance_safe(value, self.annotation):
            return value
        elif isinstance_safe(value, IntegerValidator.annotation):
            return self.annotation(value)
        elif isinstance_safe(value, FloatValidator.annotation):
            # 防止精度过长
            return self.annotation(StringValidator.annotation(value))
        try:
            maybe_str = StringValidator.maybe_str(value)
            if maybe_str:
                return self.annotation(value)
        except (UnicodeDecodeError, ValueError, TypeError, DataclassCustomError, decimal.InvalidOperation):
            raise DataclassCustomError('decimal_parsing', '输入应为有效数值')
        raise DataclassCustomError('decimal_parsing', '输入应为有效数值')


class BytesValidator(Validator):
    """字节验证器"""

    validator_name = 'bytes'
    annotation = bytes

    def validate(self, value):
        if isinstance_safe(value, self.annotation):
            return value
        elif isinstance_safe(value, StringValidator.annotation):
            return value.encode('utf-8')
        elif isinstance_safe(value, bytearray):
            return self.annotation(value)
        raise ValueError('输入应为有效字节类型')


class IsInstanceValidator(Validator):
    """目标实例验证器"""

    validator_name = 'is_instance'

    def __init__(self, annotation: _T, **kwargs):
        super().__init__(**kwargs)
        self.annotation = annotation

    def validate(self, value):
        if isinstance_safe(value, self.annotation):
            return value
        raise ValueError(f'输入应为`{_format_type(self.annotation)}`实例')

    @property
    def name(self) -> str:
        return f"{self.validator_name}[{_format_type(self.annotation)}]"


class IsSubClassValidator(Validator):
    """目标类或子类验证器"""

    validator_name = 'is_subclass'

    def __init__(self, annotation: _T, **kwargs):
        super().__init__(**kwargs)
        self.annotation = annotation

    def validate(self, value):
        if issubclass_safe(value, self.annotation):
            return value
        raise DataclassCustomError('is_subclass', f'输入应该是`{_format_type(self.annotation)}`的子类')

    @property
    def name(self) -> str:
        return f"{self.validator_name}[{_format_type(self.annotation)}]"

    @classmethod
    def build(cls, annotation: _T, **kwargs) -> 'IsSubClassValidator':
        args = get_args(annotation)
        return cls(annotation=args[0], **kwargs)


class OptionalValidator(Validator):
    """可空类型验证器"""

    validator_name = 'optional'
    annotation = optional
    validator: Validator

    def __init__(self, validator: Validator, **kwargs):
        super().__init__(**kwargs)
        self.validator = validator

    def validate(self, value):
        if value is None:
            return value
        return self.validator.validate(value)

    @classmethod
    def build(cls, annotation: _T, **kwargs):
        if not type_parser.is_optional(annotation):
            annotation = Optional[annotation]
        validator = matching_validator(get_args(annotation)[0], **kwargs)
        return cls(validator=validator, **kwargs)

    @property
    def name(self) -> str:
        return f"{self.validator_name}[{self.validator.name}]"


class UnionValidator(Validator):
    """联合类型验证器"""

    validator_name = 'union'
    annotation = Union
    validators: List[Validator]

    def __init__(self, validators: List[Validator], **kwargs):
        super().__init__(**kwargs)
        self.validators = validators

    def validate(self, value):
        err: optional[ErrorDetail] = None
        for validator in self.validators:
            errs: List[ErrorDetail] = []
            v = catch_validate_error(value, validator, [], errs)
            if not errs:
                return v
            err = errs[-1]
        raise DataclassCustomError(err.exception_type, err.msg)

    @classmethod
    def build(cls, annotation: _T, **kwargs):
        args = get_args(annotation)
        if not args:
            raise ValidatorBuildingError(f"使用Union注解至少需要有一种类型才可构建 {cls.__name__}")
        if not type_parser.is_union(annotation):
            raise ValidatorBuildingError(f"输入应为Union注解才可构建 {cls.__name__}")
        validators = [matching_validator(sub, **get_sub_validator_kwargs(kwargs, i)) for i, sub in enumerate(args)]
        return cls(validators=validators, **kwargs)

    @property
    def name(self) -> str:
        return f"{self.validator_name}[{', '.join([val.name for val in self.validators])}]"


class LiteralValidator(Validator):
    """字面量类型验证器"""

    validator_name = 'literal'
    annotation = Literal
    expected_values: Tuple[Any, ...]

    def __init__(self, *expected: Tuple[Any, ...], **kwargs):
        super().__init__(**kwargs)
        self.expected_values = expected

    def validate(self, value):
        if value in self.expected_values:
            return value
        raise ValueError(f'输入应为 {self.format_expected_values}')

    @classmethod
    def build(cls, annotation, **kwargs):
        if not type_parser.is_literal(annotation):
            raise ValueError(f"输入应为Literal注解才可构建 {cls.__name__}")
        args = get_args(annotation)
        return cls(*args)

    def __repr__(self):
        return (f"{self.__class__.__name__}("
                f"\n  name: {self.name}"
                f"\n  annotation: {self.annotation}"
                f"\n  values: {self.expected_values}"
                f"\n)")

    @property
    def name(self) -> str:
        return f'{self.validator_name}[{",".join([f"{value!r}" for value in self.expected_values])}]'

    @property
    def format_expected_values(self):
        return f'{", ".join([f"`{value!r}`" for value in self.expected_values])}'


# TODO
class FastDataclassValidator(Validator):
    """快速数据类验证器"""

    validator_name = 'fast_dataclass'

    def __init__(self, annotation: _T, **kwargs):
        super().__init__(**kwargs)
        self.annotation = annotation

    def validate(self, value):
        if isinstance_safe(value, self.annotation):
            return value
        if isinstance_safe(value, dict):
            return self.annotation(**value)
        # TODO from obj
        raise NotImplementedError('error')
        # return self.annotation

    @property
    def name(self) -> str:
        return _format_type(self.annotation)

    @classmethod
    def build(cls, annotation: _T, **kwargs) -> 'FastDataclassValidator':
        if not type_parser.is_fast_dataclass(annotation):
            raise DataclassCustomError('fast_dataclass_parsing', '输入应为快速数据类')
        return cls(annotation=annotation, **kwargs)


class PydanticModelValidator(Validator):
    """Pydantic模型验证器"""

    validator_name = 'pydantic'
    annotation = None

    def validate(self, value):
        if type_parser.is_pydantic_model(value):
            return value
        raise ValueError('输入应为Pydantic模型')

    @classmethod
    def build(cls, annotation: _T, **kwargs) -> 'PydanticModelValidator':
        try:
            from pydantic import BaseModel
        except ImportError:
            raise ImportError('导入Pydantic失败, 可能未安装')
        if not type_parser.is_pydantic_model(annotation):
            raise ValueError('输入应为Pydantic模型')
        return cls(annotation=annotation, **kwargs)


# TODO
class DataclassValidator(Validator):
    """数据类验证器"""

    validator_name = 'dataclass'

    def validate(self, value):
        try:
            from dataclasses import is_dataclass
        except ImportError:
            raise ImportError('导入Dataclass失败, 您把它删了吗？')
        if is_dataclass(value):
            return value
        raise ValueError('输入应为Dataclass数据类')


class DictValidator(Validator):
    """字典验证器"""

    validator_name = 'dict'
    annotation = dict
    key_validator: Validator
    value_validator: Validator
    min_length: optional[int]
    max_length: optional[int]

    def __init__(self, key_validator: Validator, value_validator: Validator, min_length: optional[int] = None,
                 max_length: optional[int] = None, **kwargs):
        super().__init__(**kwargs)
        self.key_validator = key_validator
        self.value_validator = value_validator
        self.min_length = min_length
        self.max_length = max_length

    def validate(self, value):
        if not isinstance_safe(value, Mapping):
            raise DataclassCustomError('dict_type', '输入应为有效键值对')
        # 验证长度
        length = len(value)
        check_collection_length(self.annotation, length, self.min_length, self.max_length)
        # 优化（双any可省略很多验证性能）
        key_validator_is_any = self.key_validator.annotation is Any
        value_validator_is_any = self.value_validator.annotation is Any
        if key_validator_is_any and value_validator_is_any:
            return value
        errs: List[ErrorDetail] = []
        # 验证keys(any优化)
        if key_validator_is_any:
            keys = value.keys()
        else:
            keys = [catch_validate_error(v, self.key_validator, [i], errs) for i, v in enumerate(value.keys())]
        # 验证values
        if value_validator_is_any:
            values = value.values()
        else:
            values = [catch_validate_error(v, self.value_validator, [i], errs) for i, v in enumerate(value.values())]
        res_dict = self.annotation(zip(keys, values))
        return res_dict

    @classmethod
    def build(cls, annotation: _T, **kwargs) -> 'DictValidator':
        _args = get_args(annotation)
        _args_length = len(_args)
        key_validator = matching_validator(_args[0], **kwargs) if _args_length >= 1 else BASE_VALIDATORS[Any]
        value_validator = matching_validator(_args[1], **kwargs) if _args_length >= 2 else BASE_VALIDATORS[Any]
        return cls(key_validator, value_validator, *_args, **kwargs)

    @property
    def name(self) -> str:
        return f'{self.validator_name}[{self.key_validator.name}, {self.value_validator.name}]'


class TypedDictField:
    name: str
    required: bool
    validator: Validator

    def __init__(self, name: str, required: bool, validator: Validator,):
        self.name = name
        self.required = required
        self.validator = validator

    def __repr__(self):
        return f"{self.__class__.__name__}(name={self.name}, required={self.required}, validator={self.validator})"


class TypedDictValidator(Validator):
    """带类型字典验证器"""

    validator_name = 'typed_dict'
    annotation = TypedDict
    fields: Dict[str, TypedDictField]

    def __init__(self, fields: Dict[str, TypedDictField], **kwargs):
        super().__init__(**kwargs)
        self.fields = fields

    def validate(self, value):
        if not isinstance_safe(value, Mapping):
            raise DataclassCustomError('dict_type', '输入应为有效键值对')
        errs: List[ErrorDetail] = []
        result: dict = {
            field_name: self.typed_dict_catch_validate_error(value, field, value.get(field_name), errs)
            for field_name, field in self.fields.items()
        }
        if errs:
            raise ValidationError(title=self.name, line_errors=errs)
        return result

    @staticmethod
    def typed_dict_catch_validate_error(origin_v, field, v, errs):
        # 必填
        if v is None and field.required:
            err = ErrorDetail(
                loc=[field.name],
                input_value=origin_v,
                exception_type='missing',
                msg='字段为必填项'
            )
            errs.append(err)
            return v
        return catch_validate_error(v, field.validator, [field.name], errs)

    @classmethod
    def build(cls, annotation: _T, **kwargs) -> 'TypedDictValidator':
        fields: Dict[str, TypedDictField] = dict()
        for k, field_name in enumerate(annotation.__annotations__):
            anno = annotation.__annotations__[field_name]
            required = True
            # 可空
            if type_parser.is_optional(anno):
                required = False
            validator = matching_validator(anno, **get_sub_validator_kwargs(kwargs))
            field = TypedDictField(field_name, required, validator)
            fields[field_name] = field
        return cls(fields=fields, **kwargs)


class ListValidator(Validator):
    """列表验证器"""

    validator_name = 'list'
    annotation = list
    item_validator: Validator
    min_length: optional[int]
    max_length: optional[int]

    def __init__(self, item_validator: Validator, min_length: optional[int] = None, max_length: optional[int] = None,
                 **kwargs):
        super().__init__(**kwargs)
        self.item_validator = item_validator
        self.min_length = min_length
        self.max_length = max_length

    def validate(self, value):
        collection = extract_collection(value, 'list_type', '列表')
        # 验证长度
        check_collection_length(self.annotation, len(collection), self.min_length, self.max_length)
        # 优化
        if self.item_validator.annotation is Any:
            is_list = isinstance_safe(collection, list)
            return collection if is_list else self.annotation(collection)
        errs: List[ErrorDetail] = []
        result: list = [
            catch_validate_error(item, self.item_validator, [i], errs)
            for i, item in enumerate(collection)
        ]
        if errs:
            raise ValidationError(title=self.name, line_errors=errs)
        return result

    @classmethod
    def build(cls, annotation: _T, **kwargs) -> 'ListValidator':
        args = get_args(annotation)
        sub_annotation = args[0] if args else Any
        item_validator = matching_validator(sub_annotation, **get_sub_validator_kwargs(kwargs))
        return cls(item_validator=item_validator, **kwargs)

    @property
    def name(self) -> str:
        return f'{self.validator_name}[{self.item_validator.name}]'


class TupleValidator(Validator):
    """元组验证器"""

    validator_name = 'tuple'
    annotation = tuple
    validators: List[Validator]
    variadic: bool = True
    min_length: optional[int]
    max_length: optional[int]

    def __init__(self, validators: List[Validator], variadic: bool = False, min_length: optional[int] = None,
                 max_length: optional[int] = None, **kwargs):
        super().__init__(**kwargs)
        self.validators = validators
        self.variadic = variadic
        self.min_length = min_length
        self.max_length = max_length

    def validate(self, value):
        collection = extract_collection(value, 'tuple_type', '元祖')
        # 检查长度
        length = len(collection)
        check_collection_length(self.annotation, length, self.min_length, self.max_length)
        errs: List[ErrorDetail] = []
        # 不是无限延伸的元祖
        if not self.variadic:
            should_be_length = len(self.validators)
            if length > should_be_length:
                raise DataclassCustomError(
                    'too_long',
                    f'验证后元组最多应包含{should_be_length}个项目，而不是{length}个'
                )
            result: list = []
            for i, val in enumerate(self.validators):
                try:
                    v = collection[i]
                    result.append(catch_validate_error(v, val, [i], errs))
                except IndexError:
                    errs.append(ErrorDetail([i], collection, 'missing', '字段为必填项'))
            if errs:
                raise ValidationError(title=self.name, line_errors=errs)
            return tuple(result)
        # 可变
        validator = self.validators[0]
        result: tuple = tuple(
            catch_validate_error(item, validator, [i], errs)
            for i, item in enumerate(collection)
        )
        if errs:
            raise ValidationError(title=self.name, line_errors=errs)
        return result

    @classmethod
    def build(cls, annotation: _T, **kwargs) -> 'TupleValidator':
        args = get_args(annotation)
        if Ellipsis in args:
            variadic = True
            args = list(args)
            args.remove(Ellipsis)
        else:
            variadic = False
        validators = [matching_validator(sub, **get_sub_validator_kwargs(kwargs, i)) for i, sub in enumerate(args)]
        if not validators:
            validators = [BASE_VALIDATORS[Any]]
            variadic = True
        if variadic and len(validators) > 1:
            raise ValidatorBuildingError(f'可变元组只能有一种类型')
        return cls(validators=validators, variadic=variadic, **kwargs)

    @property
    def name(self) -> str:
        return f'{self.validator_name}[{", ".join([val.name for val in self.validators])}]'


class SetValidator(Validator):
    """集合验证器"""

    validator_name = 'set'
    annotation = set
    item_validator: Validator
    min_length: optional[int]
    max_length: optional[int]

    def __init__(self, item_validator: optional[Validator] = None, min_length: optional[int] = None,
                 max_length: optional[int] = None, **kwargs):
        super().__init__(**kwargs)
        self.item_validator = item_validator or BASE_VALIDATORS[Any]
        self.min_length = min_length
        self.max_length = max_length

    def validate(self, value):
        collection = extract_collection(value, 'set_type', '集合')
        # 检查长度
        check_collection_length(self.annotation, len(collection), self.min_length, self.max_length)
        # 优化
        if self.item_validator.annotation is Any:
            is_set = isinstance_safe(collection, self.annotation)
            return collection if is_set else self.annotation(collection)
        errs: List[ErrorDetail] = []
        result: set = {
            catch_validate_error(item, self.item_validator, [i], errs)
            for i, item in enumerate(collection)
        }
        if errs:
            raise ValidationError(title=self.name, line_errors=errs)
        return result

    @classmethod
    def build(cls, annotation: _T, **kwargs) -> 'SetValidator':
        args = get_args(annotation)
        sub_annotation = args[0] if args else Any
        item_validator = matching_validator(sub_annotation, **get_sub_validator_kwargs(kwargs))
        return cls(item_validator=item_validator, **kwargs)

    @property
    def name(self) -> str:
        return f'{self.validator_name}[{self.item_validator.name}]'


class FrozenValidator(Validator):
    """冻结集合验证器"""

    validator_name = 'frozenset'
    annotation = frozenset
    item_validator: Validator
    min_length: optional[int]
    max_length: optional[int]

    def __init__(self, item_validator: Validator, min_length: optional[int] = None, max_length: optional[int] = None,
                 **kwargs):
        super().__init__(**kwargs)
        self.item_validator = item_validator
        self.min_length = min_length
        self.max_length = max_length

    def validate(self, value):
        collection = extract_collection(value, 'frozenset_type', '冻结集合')
        # 检查长度
        check_collection_length(self.annotation, len(collection), self.min_length, self.max_length)
        # 优化
        if self.item_validator.annotation is Any:
            is_set = isinstance_safe(collection, self.annotation)
            return collection if is_set else self.annotation(collection)
        errs: List[ErrorDetail] = []
        result: frozenset = frozenset({
            catch_validate_error(item, self.item_validator, [i], errs)
            for i, item in enumerate(collection)
        })
        if errs:
            raise ValidationError(title=self.name, line_errors=errs)
        return result

    @classmethod
    def build(cls, annotation: _T, **kwargs) -> 'FrozenValidator':
        args = get_args(annotation)
        sub_annotation = args[0] if args else Any
        item_validator = matching_validator(sub_annotation, **get_sub_validator_kwargs(kwargs))
        return cls(item_validator=item_validator, **kwargs)

    @property
    def name(self) -> str:
        return f'{self.validator_name}[{self.item_validator.name}]'


class DequeValidator(Validator):
    """队列验证器"""

    validator_name = 'deque'
    annotation = collections.deque
    item_validator: Validator
    min_length: optional[int]
    max_length: optional[int]

    def __init__(self, item_validator: Validator, min_length: optional[int] = None, max_length: optional[int] = None,
                 **kwargs):
        super().__init__(**kwargs)
        self.item_validator = item_validator
        self.min_length = min_length
        self.max_length = max_length

    def validate(self, value) -> collections.deque:
        collection = extract_collection(value, 'deque_type', '队列')
        # 检查长度
        check_collection_length(self.annotation, len(collection), self.min_length, self.max_length)
        # 优化
        if self.item_validator.annotation is Any:
            is_deque = isinstance_safe(collection, self.annotation)
            return collection if is_deque else self.annotation(collection)
        errs: List[ErrorDetail] = []
        result: collections.deque = collections.deque((
            catch_validate_error(item, self.item_validator, [i], errs)
            for i, item in enumerate(collection)
        ))
        if errs:
            raise ValidationError(title=self.name, line_errors=errs)
        return result

    @classmethod
    def build(cls, annotation: _T, **kwargs) -> 'DequeValidator':
        args = get_args(annotation)
        item_validator = matching_validator(args[0], **get_sub_validator_kwargs(kwargs)) \
            if args else BASE_VALIDATORS[Any]
        return cls(item_validator=item_validator, **kwargs)

    @property
    def name(self) -> str:
        return f'{self.validator_name}[{self.item_validator.name}]'


class GeneratorValidator(Validator):
    """生成器验证器"""

    validator_name = 'generator'
    annotation = Generator
    item_validator: Validator
    min_length: optional[int]
    max_length: optional[int]

    def __init__(self, item_validator: Validator, min_length: optional[int] = None, max_length: optional[int] = None,
                 **kwargs):
        super().__init__(**kwargs)
        self.item_validator = item_validator
        self.min_length = min_length
        self.max_length = max_length

    def validate(self, value):
        if not isinstance_safe(value, Iterable):
            raise DataclassCustomError('iterable_type', '输入应为可迭代类型')
        iterator = GeneratorIterator(
            iterable=(v for v in value),
            validator=self.item_validator,
            min_length=self.min_length,
            max_length=self.max_length,
        )
        return iterator

    @property
    def name(self) -> str:
        return f"{self.validator_name}[{_format_type(self.item_validator.name)}]"

    @classmethod
    def build(cls, annotation: _T, **kwargs) -> 'GeneratorValidator':
        args = get_args(annotation)
        sub_annotation = args[0] if args else Any
        item_validator = matching_validator(sub_annotation, **get_sub_validator_kwargs(kwargs))
        return cls(item_validator=item_validator, **kwargs)


class GeneratorIterator:
    """生成器迭代器，包含验证"""
    iterable: Generator
    validator: Validator
    min_length: optional[int]
    max_length: optional[int]
    index: int

    def __init__(self, iterable: Generator, validator: Validator, min_length: optional[int] = None,
                 max_length: optional[int] = None):
        self.iterable = iterable
        self.validator = validator
        self.min_length = min_length
        self.max_length = max_length
        self.index = 0

    def __iter__(self):
        return self.iterable

    def __next__(self):
        value = next(self.iterable)
        errs: List[ErrorDetail] = []
        value = catch_validate_error(value, self.validator, [self.index], errs)
        # 检查长度
        try:
            check_collection_length(Generator, self.index + 1, self.min_length, self.max_length)
        except DataclassCustomError as e:
            errs.append(ErrorDetail([self.index], value, e.exception_type, e.msg))
        if errs:
            raise ValidationError(title=self.__class__.__name__, line_errors=errs)
        self.index += 1
        return value

    def __repr__(self):
        return f"{self.__class__.__name__}(index={self.index}, validator={self.validator.name})"

    def __str__(self):
        return self.__repr__()


class ArgumentValidatorParameter:
    """参数验证器参数"""

    name: str
    validator: Validator


class ArgumentsValidator(Validator):
    """参数验证器"""

    validator_name = 'arguments'
    parameters: Dict[str, ArgumentValidatorParameter]
    positional_params_count: int = 0
    var_args_validator: optional[Validator]
    var_kwargs_validator: optional[Validator]

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def validate(self, value):
        pass

    @classmethod
    def build(cls, annotation: _T, **kwargs) -> 'ArgumentsValidator':
        pass


class FunctionValidator(Validator):
    """函数验证器，传递函数参数自动调用函数"""

    validator_name = 'function'
    annotation = FunctionType
    """参数验证器列表"""
    argument_validator: Validator
    function: FunctionType

    def __init__(self, function: FunctionType, **kwargs):
        super().__init__(**kwargs)
        # self.argument_validator = argument_validator
        self.function = function

    def validate(self, value):
        is_dict = isinstance(value, dict)
        if not isinstance_safe(value, (tuple, list)) and not is_dict:
            raise DataclassCustomError('arguments_type', '参数必须是元组、列表或字典')
        parameters = inspect.signature(self.function).parameters
        errs: List[ErrorDetail] = []
        for index, param_name in enumerate(parameters):
            parameter = parameters[param_name]
            try:
                if is_dict:
                    input_value = value[param_name]
                else:
                    input_value = value[index]
            except KeyError:
                err = ErrorDetail(
                    [param_name],
                    value,
                    'missing',
                    '字段不能为空'
                )
                errs.append(err)
                continue
            except IndexError:
                err = ErrorDetail(
                    [index],
                    value,
                    'missing',
                    '字段不能为空'
                )
                errs.append(err)
                continue
            if input_value is None and parameter.default == inspect.Parameter.empty:
                err = ErrorDetail(
                    [param_name],
                    value,
                    'missing',
                    '字段不能为空'
                )
                errs.append(err)
                continue
        if errs:
            raise ValidationError(title=self.name, line_errors=errs)

    def __repr__(self):
        return (f"{self.__class__.__name__}(\n  "
                f"name: {self.name!r},\n  "
                f"function: {self.function}\n  "
                f"argument_validator: {self.argument_validator}"
                f")")

    @classmethod
    def build(cls, annotation: _T, **kwargs) -> 'FunctionValidator':
        is_function = type_parser.is_function(annotation)
        if not is_function:
            raise DataclassCustomError('func_building', '输入应为函数才可构建函数验证器')
        return cls(function=annotation, **kwargs)


class CallableValidator(Validator):
    """可调用类型验证器，验证是否可调用"""

    validator_name = 'callable'
    annotation = Callable

    def validate(self, value):
        try:
            if callable(value):
                return value
        except TypeError:
            raise DataclassCustomError('callable_type', '输入应为可调用类型')
        raise DataclassCustomError('callable_type', '输入应为可调用类型')


class DatetimeValidator(Validator):
    """日期时间验证器"""

    validator_name = 'datetime'
    annotation = datetime.datetime

    """年长度"""
    year_length = len(StringValidator.annotation(datetime.MAXYEAR))
    """月长度"""
    month_length = 2
    """日长度"""
    day_length = 2
    """总长度"""
    date_length = year_length + month_length + day_length
    """时间戳容忍值"""
    tolerance_value = 100
    """对比时间戳"""
    compare_timestamp = time.time()
    """时间戳长度"""
    timestamp_length = len(StringValidator.annotation(int(compare_timestamp)))

    def validate(self, value):
        if isinstance_safe(value, self.annotation):
            return value
        elif isinstance_safe(value, DateValidator.annotation):
            return self.date_to_datetime(value)
        try:
            if isinstance_safe(value, (IntegerValidator.annotation, FloatValidator.annotation)):
                return self.str_or_int_to_datetime(IntegerValidator.annotation(value))
            maybe_str = StringValidator.maybe_str(value, raise_error=False)
            if maybe_str:
                return self.str_or_int_to_datetime(maybe_str)
        except DataclassCustomError as e:
            raise e
        except (ValueError, TypeError) as e:
            raise self.get_default_error(StringValidator.annotation(e))
        raise self.get_default_error()

    @classmethod
    def get_default_error(cls, msg: optional[str] = None):
        return DataclassCustomError('datetime_parsing', msg or '输入应为有效的日期时间或日期')

    @classmethod
    def timestamp_to_datetime(cls, value: Union[int, float]) -> datetime.datetime:
        # 兼容毫秒级时间戳 下方为了兼容'1718245600000.0' 这种字符串带小数点的情况无法直接int
        if len(str(int(float(value)))) > cls.timestamp_length:
            value = value / 1000
        return datetime.datetime.fromtimestamp(value)

    @classmethod
    def date_to_datetime(cls, value: datetime.date) -> datetime.datetime:
        return datetime.datetime(year=value.year, month=value.month, day=value.day)

    @classmethod
    def str_or_int_to_datetime(cls, value: Union[str, int]):
        value = StringValidator.annotation(value)
        length = len(value)
        if length < cls.year_length:
            raise DataclassCustomError('datetime_parsing', '输入应为有效的日期时间或日期，输入太短')
        int_value = None
        try:
            int_value = int(float(value))  # 兼容'2024.0'
        except (ValueError, TypeError):
            pass
        if int_value and int_value < datetime.MAXYEAR:
            # e.g: 2024 4位最短
            return datetime.datetime(int_value, 1, 1)
        elif length == cls.date_length and int_value:
            # e.g: 20240204 8位无符号版本
            year = int(value[:cls.year_length])
            month = int(value[cls.year_length:6])
            day = int(value[cls.year_length + 2:])
            return datetime.datetime(year, month, day)
        elif length == cls.date_length + 2 and int_value:
            # e.g: 1718245600 时间戳10位版本
            return datetime.datetime.fromtimestamp(int_value)
        elif length == cls.date_length + 2 and int_value is None:
            # e.g: 2024-02-04 10位版本
            delimiter = cls.get_delimiter(value)
            __format = f'%Y{delimiter}%m{delimiter}%d'
            try:
                return datetime.datetime.strptime(value, __format)
            except (ValueError, TypeError):
                raise DataclassCustomError('date_parsing', f'时间数据“{value}”与格式“{__format}”不匹配')
        elif length == cls.date_length + 5 and int_value:
            # e.g: 1718245600000 时间戳13位版本
            return cls.timestamp_to_datetime(int_value)
        elif cls.date_length + 10 >= length >= cls.date_length + 6 and int_value:
            # e.g: 17182456.000 时间戳14-18位版本
            return cls.timestamp_to_datetime(int_value)
        elif length == cls.date_length + 8 and int_value is None:
            # e.g: 2024-02-04 10:15 16位版本
            delimiter = cls.get_delimiter(value)
            __format = f'%Y{delimiter}%m{delimiter}%d %H:%M'
            try:
                return datetime.datetime.strptime(value, __format)
            except (ValueError, TypeError):
                raise DataclassCustomError(
                    'datetime_parsing',
                    f'时间数据“{value}”与格式“{__format}”不匹配'
                )
        elif length == cls.date_length + 11 and int_value is None and 'T' not in value:
            # e.g: 2024-02-04 10:15:30
            delimiter = cls.get_delimiter(value)
            __format = f'%Y{delimiter}%m{delimiter}%d %H:%M:%S'
            try:
                return datetime.datetime.strptime(value, __format)
            except (ValueError, TypeError):
                raise DataclassCustomError(
                    'datetime_parsing',
                    f'时间数据“{value}”与格式“{__format}”不匹配'
                )
        elif length >= cls.date_length + 11 and int_value is None:
            # rfc3339格式支持
            try:
                return datetime.datetime.fromisoformat(value)
            except (ValueError, TypeError):
                raise DataclassCustomError(
                    'datetime_from_rfc3339_parsing',
                    '输入的RFC3339日期时间格式有误'
                )
        raise DatetimeValidator.get_default_error()

    @staticmethod
    def get_delimiter(value: str):
        return '/' if '/' in value else '.' if '.' in value else '-'


class TimeValidator(Validator):
    """时间验证器"""

    validator_name = 'time'
    annotation = datetime.time
    """模式：second模式数字当秒处理，time模式数字将转换时间，默认`second`"""
    mode: Literal['second', 'time']

    def __init__(self, mode: Literal['second', 'time'] = 'second', **kwargs):
        super().__init__(**kwargs)
        self.mode = mode

    def validate(self, value):
        if isinstance_safe(value, self.annotation):
            return value
        if isinstance_safe(value, DatetimeValidator.annotation):
            return value.time()
        elif isinstance_safe(value, IntegerValidator.annotation):
            return self.int_to_time(value)
        elif isinstance_safe(value, FloatValidator.annotation):
            return self.int_to_time(IntegerValidator.annotation(value))

        maybe_str = StringValidator.maybe_str(value, raise_error=False)
        if maybe_str:
            return self.str_to_time(maybe_str)
        raise self.get_default_error()

    def int_to_time(self, value: int) -> datetime.time:
        if self.mode == 'second':
            hour = value // 3600
            minute = value // 60
            second = value % 60
            return datetime.time(hour, minute, second)
        str_value = str(value)
        length = len(str_value)
        if length == 3:
            return datetime.time(int(str_value[0]), int(str_value[1:]), 0)
        elif length == 4:
            return datetime.time(int(str_value[:2]), int(str_value[2:]), 0)
        raise self.get_default_error()

    @classmethod
    def str_to_time(cls, value: str) -> datetime.time:
        length = len(value)
        try:
            if length >= 5:
                return datetime.time.fromisoformat(value)
        except (ValueError, TypeError):
            raise DataclassCustomError(
                'time_from_rfc3339_parsing',
                '输入应为时间类型，无法解析异常的RFC时间格式'
            )
        raise cls.get_default_error()

    @classmethod
    def get_default_error(cls, msg: optional[str] = None):
        return DataclassCustomError('time_parsing', msg or '输入应为有效时间类型')


class TimedeltaValidator(Validator):
    """时间增量验证器"""

    validator_name = 'timedelta'
    annotation = datetime.timedelta

    def validate(self, value):
        if isinstance_safe(value, self.annotation):
            return value
        elif isinstance_safe(value, (IntegerValidator.annotation, FloatValidator.annotation)):
            return self.annotation(seconds=value)
        maybe_str = StringValidator.maybe_str(value)
        if maybe_str:
            return self.str_to_timedelta(maybe_str)
        raise ValueError("输入应为有效时间增量类型")

    @classmethod
    def str_to_timedelta(cls, value: str):
        # Define regex patterns for both formats
        pattern_1 = re.compile(r'^(-)?(?:(\d+)d,)?(?:(\d+)d)?(\d+):(\d+):(\d+)(?:\.(\d+))?$', re.IGNORECASE)
        pattern_2 = re.compile(r'^([+-])?P?(?:(\d+)D)?(?:T(?:(\d+)H)?(?:(\d+)M)?(?:(\d+)S)?)?$', re.IGNORECASE)

        # Try matching the first format
        match_1 = pattern_1.match(value)
        if match_1:
            sign, days_1, days_2, hours, minutes, seconds, microseconds = match_1.groups()
            days = int(days_1) if days_1 else (int(days_2) if days_2 else 0)
            hours = int(hours) if hours else 0
            minutes = int(minutes) if minutes else 0
            seconds = int(seconds) if seconds else 0
            microseconds = int(microseconds.ljust(6, '0')) if microseconds else 0

            delta = cls.annotation(days=days, hours=hours, minutes=minutes, seconds=seconds, microseconds=microseconds)
            return -delta if sign else delta

        # Try matching the second format
        match_2 = pattern_2.match(value)
        if match_2:
            sign, days, hours, minutes, seconds = match_2.groups()
            days = int(days) if days else 0
            hours = int(hours) if hours else 0
            minutes = int(minutes) if minutes else 0
            seconds = int(seconds) if seconds else 0

            delta = cls.annotation(days=days, hours=hours, minutes=minutes, seconds=seconds)
            return -delta if sign == '-' else delta
        raise ValueError('输入应为有效时间增量类型')


class DateValidator(Validator):
    """日期验证器"""

    annotation = datetime.date

    def validate(self, value):
        if isinstance_safe(value, self.annotation):
            return value
        elif isinstance_safe(value, DatetimeValidator.annotation):
            return self.datetime_to_date(value)
        try:
            if isinstance_safe(value, (StringValidator.annotation, FloatValidator.annotation,
                                       IntegerValidator.annotation)):
                return DatetimeValidator.str_or_int_to_datetime(value).date()
        except ValueError:
            raise ValueError("输入应为有效日期类型")
        raise ValueError("输入应为有效日期类型")

    @staticmethod
    def datetime_to_date(value: datetime.datetime) -> datetime.date:
        return value.date()


class IntEnumValidator(Validator):
    """整型枚举验证器"""

    validator_name = 'int_enum'
    annotation = enum.IntEnum
    enum_class: enum.EnumType
    values: list

    def __init__(self, enum_class: enum.EnumType, **kwargs):
        super().__init__(**kwargs)
        self.enum_class = enum_class
        self.values = [i.value for i in self.enum_class]

    def validate(self, value):
        if isinstance_safe(value, self.annotation):
            return value
        try:
            return self.enum_class(int(value))
        except (ValueError, TypeError):
            raise DataclassCustomError('int_enum_parsing', f'输入应为 {self.should_input_texts}')

    @property
    def name(self) -> str:
        return f"{self.validator_name}[{_format_type(self.enum_class)}]"

    @property
    def should_input_texts(self) -> str:
        return f'{",".join([f"`{value}`" for value in self.values])}'

    @classmethod
    def build(cls, annotation: _T, **kwargs) -> 'IntEnumValidator':
        if not issubclass_safe(annotation, enum.IntEnum):
            raise DataclassCustomError('building_error', '输入应为IntEnum类型才可构建验证器')
        return cls(annotation, **kwargs)


class EnumValidator(Validator):
    """枚举验证器"""

    validator_name = 'enum'
    annotation = enum.Enum
    enum_class: enum.EnumType
    use_value: bool

    def __init__(self, enum_class: enum.EnumType, use_value: bool = True, **kwargs):
        super().__init__(**kwargs)
        self.enum_class = enum_class
        self.use_value = use_value
        self.values = [i.value if self.use_value else i.name for i in self.enum_class]

    def validate(self, value):
        if isinstance_safe(value, self.enum_class):
            return value
        # 很优的方案，intEnum传递为str时也正确转换
        try:
            if not self.use_value:
                maybe_str = StringValidator.maybe_str(value, raise_error=False)
                return self.enum_class[maybe_str]
            try:
                return self.enum_class(value)
            except ValueError:
                return self.enum_class(int(value))
        except (KeyError, ValueError, TypeError):
            pass
        raise DataclassCustomError('enum_parsing', f'输入应为 {self.should_input_texts}')

    @classmethod
    def build(cls, annotation: _T, **kwargs) -> 'EnumValidator':
        if not issubclass_safe(annotation, enum.Enum):
            raise DataclassCustomError('building_error', '输入应为枚举类型才可构建验证器')
        return cls(annotation, **kwargs)

    @property
    def name(self) -> str:
        return f'{self.validator_name}[{_format_type(self.enum_class)}]'

    @property
    def should_input_texts(self) -> str:
        return f'{",".join([f"`{value}`" for value in self.values])}'


class UuidValidator(Validator):
    """UUID类型验证器"""

    validator_name = 'uuid'
    annotation = uuid.UUID
    version: optional[int]

    def __init__(self, version: optional[int] = None, **kwargs):
        super().__init__(**kwargs)
        self.version: optional[int] = version

    def validate(self, value) -> uuid.UUID:
        try:
            if isinstance_safe(value, self.annotation):
                self.check_version(value, self.version)
                return value
            maybe_str = StringValidator.maybe_str(value, raise_error=False)
            if maybe_str:
                return self.str_to_uuid(maybe_str)
        except DataclassCustomError as e:
            raise e
        except (ValueError, TypeError):
            pass
        raise DataclassCustomError('uuid_parsing', '输入应为有效的UUID')

    @staticmethod
    def check_version(value: uuid.UUID, version: optional[int]):
        if version and value.version != version:
            raise DataclassCustomError('uuid_version', f'输入的UUID值版本错误，应为UUID版本`{version}`')
        return True

    def str_to_uuid(self, value: str) -> uuid.UUID:
        res = uuid.UUID(value)
        self.check_version(res, self.version)
        return res


def matching_validator(annotation: _T, **kwargs):
    """匹配验证器"""
    origin_annotation = type_parser.get_origin_safe(annotation) or annotation
    # Optional原型为Union不放在上面会变成Union验证器
    if type_parser.is_optional(annotation):
        return OptionalValidator.build(annotation, **kwargs)
    try:
        return MATCH_VALIDATOR[origin_annotation].build(annotation, **kwargs)
    except KeyError:
        # 特殊注解
        if issubclass_safe(annotation, enum.IntEnum):
            return MATCH_VALIDATOR[enum.IntEnum].build(annotation, **kwargs)
        elif issubclass_safe(annotation, enum.Enum):
            return MATCH_VALIDATOR[enum.Enum].build(annotation, **kwargs)
        # typing
        elif is_typeddict(annotation):
            return MATCH_VALIDATOR[TypedDict].build(annotation, **kwargs)
        # typing_extensions
        try:
            if typing_extensions.is_typeddict(annotation):
                return MATCH_VALIDATOR[TypedDict].build(annotation, **kwargs)
        except ImportError:
            pass
        return IsInstanceValidator.build(annotation, **kwargs)


def catch_validate_error(
        v,
        val: Validator,
        loc: List[Any],
        errs: List[ErrorDetail],
        exception_type: optional[str] = None,
        errmsg: optional[str] = None
):
    """捕捉异常"""
    try:
        return val.validate(v)
    except ValidationError as e:
        for error in e.errors():
            error.loc.insert(0, *loc)
        errs.extend(e.line_errors)
    except DataclassCustomError as e:
        error = ErrorDetail(
            loc=loc,
            input_value=v,
            exception_type=e.exception_type,
            msg=e.msg,
        )
        errs.append(error)
    except (ValueError, TypeError) as e:
        error = ErrorDetail(
            loc=loc,
            input_value=v,
            exception_type=exception_type or type(e),
            msg=errmsg or str(e),
        )
        errs.append(error)
    except Exception as e:
        error = ErrorDetail(
            loc=loc,
            input_value=v,
            exception_type=exception_type or 'exception_error',
            msg=errmsg or str(e),
        )
        errs.append(error)
    return v


def check_collection_length(annotation, length: int, min_length: int, max_length: int):
    """检查集合长度"""
    annotation_texts = {
        list: '列表',
        tuple: '元祖',
        dict: '字典',
        set: '集合',
        frozenset: '冻结集合',
        Collection: '集合',
        Iterable: '可迭代',
        Generator: '生成器',
        collections.deque: '队列'
    }
    annotation_text = annotation_texts.get(annotation, '集合')
    if min_length is not None and length < min_length:
        raise DataclassCustomError(
            'iter_too_short',
            f'{annotation_text}最小应包含{min_length}项，而不是{length}项'
        )
    elif max_length is not None and length > max_length:
        raise DataclassCustomError(
            'iter_too_long',
            f'{annotation_text}最多应包含{max_length}项，而不是{length}项'
        )


def extract_collection(v, exception_type: str = 'collection_type', type_text: str = '集合'):
    """尝试将其作为一个可迭代可获取长度的东西，但排除字符串和映射类型"""
    if isinstance_safe(v, (list, tuple, set, frozenset)):
        return v
    elif not isinstance_safe(v, (str, bytes, bytearray, dict, Mapping)) and isinstance(v, Collection):
        return v
    raise DataclassCustomError(exception_type, f'输入应为{type_text}类型')


BASE_VALIDATORS = {
    Any: AnyValidator(),
    str: StringValidator(),
    bool: BoolValidator(),
    int: IntegerValidator(),
    float: FloatValidator(),
    bytes: BytesValidator(),
    Decimal: DecimalValidator(),
    datetime.datetime: DatetimeValidator(),
    datetime.date: DateValidator(),
    datetime.time: TimeValidator(),
    datetime.timedelta: TimedeltaValidator(),
    uuid.UUID: UuidValidator(),
}

MATCH_VALIDATOR = {
    Any: AnyValidator,
    str: StringValidator,
    bool: BoolValidator,
    int: IntegerValidator,
    float: FloatValidator,
    bytes: BytesValidator,
    Decimal: DecimalValidator,
    datetime.datetime: DatetimeValidator,
    datetime.date: DateValidator,
    datetime.time: TimeValidator,
    datetime.timedelta: TimedeltaValidator,
    enum.Enum: EnumValidator,
    enum.StrEnum: EnumValidator,
    enum.IntEnum: IntEnumValidator,
    uuid.UUID: UuidValidator,
    Collection: ListValidator,
    collections.abc.Collection: ListValidator,
    Sequence: ListValidator,
    collections.abc.Sequence: ListValidator,
    list: ListValidator,
    List: ListValidator,
    tuple: TupleValidator,
    Tuple: TupleValidator,
    set: SetValidator,
    Set: SetValidator,
    collections.abc.Set: SetValidator,
    frozenset: FrozenValidator,
    FrozenSet: FrozenValidator,
    collections.abc.Mapping: DictValidator,
    dict: DictValidator,
    Dict: DictValidator,
    collections.deque: DequeValidator,
    Deque: DequeValidator,
    Literal: LiteralValidator,
    Optional: OptionalValidator,
    Union: UnionValidator,
    Iterable: GeneratorValidator,
    collections.abc.Iterable: GeneratorValidator,
    Generator: GeneratorValidator,
    collections.abc.Generator: GeneratorValidator,
    Type: IsSubClassValidator,
    type: IsSubClassValidator,
    TypedDict: TypedDictValidator,
}


# -*- coding:utf-8 -*-
import _thread
import functools
from typing import Union

from .constants import _SUB_VALIDATOR_KWARGS_NAME


def _recursive_repr(user_function):
    # Decorator to make a repr function return "..." for a recursive
    # call.
    repr_running = set()

    @functools.wraps(user_function)
    def wrapper(self):
        key = id(self), _thread.get_ident()
        if key in repr_running:
            return '...'
        repr_running.add(key)
        try:
            result = user_function(self)
        finally:
            repr_running.discard(key)
        return result

    return wrapper


def _format_type(_type) -> str:
    if hasattr(_type, "__name__"):
        return _type.__name__
    elif hasattr(_type, "__class__"):
        return _type.__class__.__name__
    type_str = str(_type)
    type_str = type_str.replace("<class '", "")
    type_str = type_str if not type_str.endswith("'>") else type_str[:-2]
    return type_str


def camel_to_snake(text: str) -> str:
    """驼峰转下划线"""
    return ''.join(['_' + i.lower() if i.isupper() else i for i in text]).lstrip('_')


def snake_to_camel(text: str) -> str:
    """下划线转驼峰"""
    return ''.join(w.capitalize() for w in text.split('_'))


def isinstance_safe(v, tp) -> bool:
    """安全的检查类型"""
    try:
        return isinstance(v, tp)
    except (Exception,):
        return False


def issubclass_safe(v, tp) -> bool:
    try:
        return issubclass(v, tp)
    except (Exception,):
        return False


def get_sub_validator_kwargs(validator_kwargs: Union[dict, list], index: int = 0) -> dict:
    try:
        sub_validator_kwargs = validator_kwargs[_SUB_VALIDATOR_KWARGS_NAME]
        if isinstance(sub_validator_kwargs, dict):
            del validator_kwargs[_SUB_VALIDATOR_KWARGS_NAME]
            return sub_validator_kwargs
        else:
            kwargs = sub_validator_kwargs[index]
            if index == len(sub_validator_kwargs) - 1:
                validator_kwargs.pop(_SUB_VALIDATOR_KWARGS_NAME)
            return kwargs
    except (KeyError,):
        return dict()



from setuptools import setup

setup(
    name='fast_serializer',
    version='0.8.1',
    author='且听风铃、我是指针*、ZDLAY、KYZ',
    author_email='breezechime@163.com',
    description='python 数据类验证器和序列化框架',
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    url='https://github.com/breezechime/fast-serializer',
    license='MIT',
    packages=['fast_serializer'],
    install_requires=[],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
    ],
    python_requires='>=3.6',
)


# -*- coding:utf-8 -*-
from typing import Dict, Any


class JsonSchema:
    """JSON schemas"""

    schema_draft = 'https://json-schema.org/draft/2020-12/schema'

    def __init__(self):
        self.schema_definition: Dict[str, Any] = {}

    @classmethod
    def generate(cls, dataclass):
        pass


# -*- coding:utf-8 -*-
import json
from typing import (
    Any, List, Union
)
from .types import optional
from .utils import camel_to_snake, _format_type, isinstance_safe


def _format_exception_type(exception_type: Union[str, type]) -> str:
    if isinstance_safe(exception_type, str):
        return exception_type
    return camel_to_snake(exception_type.__class__.__name__)


class ErrorDetail:
    """错误详情"""

    """键名"""
    # key: str

    """键名索引列表"""
    loc: List[Any]

    """输入值"""
    input_value: Any

    """异常类型"""
    exception_type: str

    """错误信息"""
    msg: str

    """上下文信息"""
    ctx: optional[dict[str, Any]]

    __slots__ = ('loc', 'input_value', 'exception_type', 'msg', 'ctx')

    def __init__(self, loc: List[Any], input_value: Any, exception_type: Union[str, type], msg: str,
                 ctx: optional[dict[str, Any]] = None):
        # self.key = key
        self.loc = loc
        self.input_value = input_value
        self.exception_type = _format_exception_type(exception_type)
        self.msg = msg
        self.ctx = ctx

    def __str__(self):
        return json.dumps(self.__dict__(), indent=4, ensure_ascii=False)

    def __repr__(self):
        return self.__str__()

    def __dict__(self):
        return {
            # 'key': self.key,
            'loc': self.loc,
            'input_value': self.input_value,
            'exception_type': self.exception_type,
            'msg': self.msg,
            'ctx': self.ctx
        }


class ValidationError(ValueError):
    title: str
    line_errors: List[ErrorDetail]

    def __init__(self, title: str, line_errors: List[ErrorDetail], *args):
        super().__init__(*args)
        self.title = title
        self.line_errors = line_errors or []

    def errors(self):
        return self.line_errors

    def error_count(self) -> int:
        return len(self.line_errors)

    def json(self) -> str:
        return json.dumps(self.__dict__(), indent=4, ensure_ascii=False)

    def __str__(self):
        plural = 's' if len(self.line_errors) > 1 else ''
        return f'{len(self.line_errors)} validation error{plural} for {self.title}\n{self.format_line_errors()}'

    def __repr__(self):
        return self.__str__()

    def __dict__(self):
        return [error.__dict__() for error in self.line_errors]

    def format_line_errors(self):
        text = ""
        length = len(self.line_errors)
        for index, error in enumerate(self.line_errors):
            enter_line = '' if index == length - 1 else '\n'
            end_text = (f"["
                        f"exception_type={error.exception_type}, "
                        f"input_value={error.input_value!r}, "
                        f"input_type={_format_type(error.input_value)}"
                        f"]")
            text += f"{'.'.join([str(e) for e in error.loc])}\n  {error.msg} {end_text}{enter_line}"
        return text


class DataclassCustomError(ValueError):
    exception_type: str
    msg: str
    context: optional[dict]

    def __init__(self, exception_type: str, msg: str, context: optional[dict] = None, *args):
        super().__init__(*args)
        self.exception_type = exception_type
        self.msg = msg
        self.context = context

    def message(self):
        return self.format_message(self.msg, self.context)

    def __str__(self):
        return self.message()

    def __repr__(self):
        msg = self.message()
        if self.context is not None:
            context_repr = {k: repr(v) for k, v in self.context.items()}
            return f"{msg} [type={self.error_type}, context={context_repr}]"
        else:
            return f"{msg} [type={self.error_type}, context=None]"

    @staticmethod
    def format_message(msg: str, context: optional[str]) -> str:
        if not context:
            return msg
        message = msg
        for key, value in context.items():
            placeholder = f"{{{key}}}"
            if isinstance(value, str):
                message = message.replace(placeholder, value)
            elif isinstance(value, int):
                message = message.replace(placeholder, str(value))
            else:
                # fallback for anything else just in case
                message = message.replace(placeholder, str(value))
        return message


class ValidatorBuildingError(TypeError):

    def __init__(self, msg: str, *args):
        super().__init__(*args)
        self.msg = msg

    def __str__(self):
        return self.msg

    def __repr__(self):
        return self.__str__()


# -*- coding:utf-8 -*-
from .globals import GlobalSetting
from .dataclass_config import DataclassConfig
from .field import Field, field
from .type_parser import TypeParser
from .automation import getter
from .exceptions import ValidationError, ErrorDetail, DataclassCustomError
from .decorators import (field_serializer, field_validator, type_serializer, type_validator,
                         dataclass_serializer, dataclass_validator)
from .fast_dataclass import FastDataclass


__all__ = [
    'GlobalSetting',
    'DataclassConfig',
    'Field',
    'field',
    'TypeParser',
    'getter',
    'ValidationError',
    'ErrorDetail',
    'DataclassCustomError',
    'FastDataclass',
    'field_serializer',
    'field_validator',
    'type_serializer',
    'type_validator',
    'dataclass_serializer',
    'dataclass_validator'
]